/* ----------------------------------------------------------------- */
/* PROGRAM  buffer   :                                               */
/*    This is the bounded buffer problem                             */
/* ----------------------------------------------------------------- */

#include  <stdio.h>
#include  <stdlib.h>
#include  "mtuThread.h"

/* ----------------------------------------------------------------- */
/*                        Function Prototypes                        */
/* ----------------------------------------------------------------- */

void  Producer(int);
void  Consumer(int);

/* ----------------------------------------------------------------- */
/*                              Channel                              */
/* ----------------------------------------------------------------- */

CHANNEL_t Chan;

/* ----------------------------------------------------------------- */
/* FUNCTION  Producer:                                               */
/*    Function Producer keeps sending data to a mailbox.             */
/* ----------------------------------------------------------------- */

void  Producer(int  ID)
{
     int  i, data;

     for (i = 0; ; i++) {
          data = ID*1000 + i;
          CHAN_SEND(Chan, &data, sizeof(int));
          printf("Producer %d has sent data %d\n", ID, data);
          THREAD_YIELD();
     }

     THREAD_EXIT();
}

/* ----------------------------------------------------------------- */
/* FUNCTION  Consumer:                                               */
/*    Function Consumer keeps receiving mail from the mailbox.       */
/* ----------------------------------------------------------------- */

void  Consumer(int ID)
{
     int *data, size;

     while (1) {
          CHAN_RECV(Chan, (void**) &data, &size);
          printf("     Consumer %d received %d\n", ID, *data);
          THREAD_YIELD();
     }

     THREAD_EXIT();
}

/* ----------------------------------------------------------------- */
/*                    The main program starts here                   */
/* ----------------------------------------------------------------- */

int  main(int argc, char *argv[])
{
     THREAD_t   Producer1, Producer2, Consumer1, Consumer2;
     int        i;
     char       ch;

     printf("Use Ctrl-C to kill this program\n");
     printf("Hit any key to continue... ");
     ch = getchar();
     printf("\n");

     Chan = CHAN_INIT("Channel", ASYNCHRONOUS, MANYTOMANY);

     Producer1 = THREAD_CREATE(Producer, THREAD_SIZE, THREAD_NORMAL, 
			       1, (char **) 0);
     Producer2 = THREAD_CREATE(Producer, THREAD_SIZE, THREAD_NORMAL, 
			       2, (char **) 0);
     Consumer1 = THREAD_CREATE(Consumer, THREAD_SIZE, THREAD_NORMAL, 
			       1, (char **) 0);
     Consumer2 = THREAD_CREATE(Consumer, THREAD_SIZE, THREAD_NORMAL, 
			       2, (char **) 0);

     THREAD_JOIN(Producer1);

     return 0;
}
