/* ---------------------------------------------------------------- */
/* FILE   : channel.c
/* AUTHOR : J.R. Lewis III
/* INFO   : This is a mini-example showing how the channel 
/*          primitive is used. This illustrates how we can use CHAN_SEND
/*          to send more than one word of information through a 
/*          channel.
/* NOTE	  : This file is slightly modified by TinTin Yu in order to use
/*			the Asychronous-ManyToMany Channel. (Nov 16 2005)
/* ---------------------------------------------------------------- */

#include <stdio.h>
#include <string.h>
#include "mtuThread.h"

CHANNEL_t C;

void Thread1(int ID) {
	char buffer[20];

	sprintf(&buffer[0], "Test Message");

	printf("\tThread 1: Sending \"%s\"\n", buffer);

	/* Note that strlen()+1 is necessary for
	   copying the trailing null-terminator */
	CHAN_SEND(C, buffer, (strlen(buffer)+1)*sizeof(char));

	printf("\tThread 1: Message successfully sent!\n");

	THREAD_EXIT();
}

void Thread2(int ID) {
	char *buffer;
	int size;

	printf("\t\tThread 2: Waiting to receive.\n", buffer);

	CHAN_RECV(C, (void*) &buffer, &size);
	
	printf("\t\tThread 2: Received \"%s\"\n", buffer);

	THREAD_EXIT();
}

int main(int argc, char **argv) {
	THREAD_t T[2];

	T[0] = THREAD_CREATE(Thread1,THREAD_SIZE,THREAD_SUSPENDED,1,(char**) 0);
	T[1] = THREAD_CREATE(Thread2,THREAD_SIZE,THREAD_SUSPENDED,2,(char**) 0);

	C = CHAN_INIT("Channel1", ASYNCHRONOUS, MANYTOMANY);
	printf("Main: Channel is initialized\n");

	THREAD_RESUME(T[0]);
	THREAD_RESUME(T[1]);

	printf("Main: Waiting to join\n");
	THREAD_JOIN(T[0]);
	THREAD_JOIN(T[1]);

    CHAN_DESTROY(C);
	printf("Main: Done\n");

	return 0;
}
